/******************************************************************************
* Filename : gpio.c
* This part is used to control LED and detect button-press
* 
******************************************************************************/

#include <common.h>


#define SYSCTR_ADDR	0xB0000000
#define IRQ_ADDR 	0xB0000200
#define PRGIO_ADDR 	0xB0000600
#define GPIO_0                    (0x0001)/*EZSetup(K20)*/
#define GPIO_7                    (0x0080)/*Ready LED(T18)*/
#define GPIO_10                   (0x0400)/*reset default*/

#define REG_PIODIR		(PRGIO_ADDR + 0x24)
#define REG_PIODATA		(PRGIO_ADDR + 0x20)
#define REG_PIOPOL		(PRGIO_ADDR + 0x28)
#define REG_GPIOMODE		(SYSCTR_ADDR + 0x60)
#define RALINK_GPIOMODE_DFT     0x1C

#define REG_INTENA		(IRQ_ADDR + 0x34)
#define REG_INTDIS		(IRQ_ADDR + 0x38)

#define RESET_BTN_BIT			GPIO_10	// rt-n12u
#define EZSETUP_BTN_BIT			GPIO_0
#define READY_LED_BIT			GPIO_7

#define SYSCFG_OFFSET		0x10
#define UARTF_PCM_MODE_SHIFT	6
/*
6 R/W UARTF_PCM_MODE
0: Set 4 of UART-Full pins as normal UART function
1: Set 4 of UART-Full pins as normal PCM function
1b0
*/
#define GPIOMODE_OFFSET		0x60
#define UARTF_GPIO_MODE_SHIFT	1
/*
1 R/W UARTF_GPIO_MODE
0:Normal Mode
1:GPIO Mode
Control GPIO[6:3]
1b1
*/
void gpio_init();
void LEDON(void);
void LEDOFF(void);

void uart_pcm_mode(unsigned x)
{
	if (0 == x)
	{
		le32_to_cpu(*(volatile u_long *)(SYSCTR_ADDR + SYSCFG_OFFSET))&=~(1<<UARTF_PCM_MODE_SHIFT);
	}
	else
		if(1 == x)
		{
			le32_to_cpu(*(volatile u_long *)(SYSCTR_ADDR + SYSCFG_OFFSET))|= (1<<UARTF_PCM_MODE_SHIFT);
		}
}


void uart_gpio_mode(unsigned x)
{
	if (0 == x)
	{
		le32_to_cpu(*(volatile u_long *)(SYSCTR_ADDR + GPIOMODE_OFFSET))&=~(1<<UARTF_GPIO_MODE_SHIFT);
	}
	else 
		if(1 == x)
		{
			le32_to_cpu(*(volatile u_long *)(SYSCTR_ADDR + GPIOMODE_OFFSET))|= (1<<UARTF_GPIO_MODE_SHIFT);
		}
}


/*
set the direction of the gpio
0:input
1:output
*/

void set_gpio_dir(int dir,int nGpio)
{
	if (0 == dir)
	{
		//le32_to_cpu(*(volatile u_long *)REG_PIODIR)&=~nGpio;
		*(volatile u32 *)(REG_PIODIR) &= cpu_to_le32(~nGpio);
		//*(volatile u32 *)(REG_PIODIR) &= cpu_to_le32(0);
	}
	else 
		if(1 == dir)
		{
			//le32_to_cpu(*(volatile u_long *)REG_PIODIR)|=nGpio;
			*(volatile u32 *)(REG_PIODIR) |= cpu_to_le32(nGpio);
		}
}

void gpio_init()
{
        unsigned long gpiomode;

	//printf("gpio init\n");	// tmp test
        gpiomode = le32_to_cpu(*(volatile u32 *)(REG_GPIOMODE));
        gpiomode &= ~0x1C;  //clear bit[2:4]UARTF_SHARE_MODE
        gpiomode |= RALINK_GPIOMODE_DFT;
        *(volatile u32 *)(REG_GPIOMODE) = cpu_to_le32(gpiomode);

        set_gpio_dir(0,RESET_BTN_BIT);
        set_gpio_dir(0,EZSETUP_BTN_BIT);
        //set_gpio_dir(1,READY_LED_BIT);
}

void led_control(int state)
{
	
	if(1 == state)	// off
	{
		le32_to_cpu(*(volatile u_long *)REG_PIODATA)|=READY_LED_BIT;
		//le32_to_cpu(*(volatile u_long *)REG_PIOPOL)&=~READY_LED_BIT;
		set_gpio_dir(0,READY_LED_BIT);//set the corresponding gpio pin as output
	}
	else	// on
	{
		set_gpio_dir(1,READY_LED_BIT);//set the corresponding gpio pin as output
		le32_to_cpu(*(volatile u_long *)REG_PIODATA)&=~READY_LED_BIT;
		//le32_to_cpu(*(volatile u_long *)REG_PIOPOL)|=READY_LED_BIT;
	}
}

unsigned long DETECT(void)
{
	unsigned long tmp0;
	unsigned long tmp3, tmp1, tmp2;

	tmp0 = le32_to_cpu(*(volatile u_long *)REG_PIODATA);

	//tmp1 = le32_to_cpu(*(volatile u_long *)REG_PIODIR);
	//tmp2 = le32_to_cpu(*(volatile u_long *)REG_PIOPOL);

	//printf("before LEDON:\n");	 // tmp test
	//printf("data:%x, dir:%x, pol:%x\n", tmp0, tmp1, tmp2);

	//printf("test LEDON\n");
	//LEDON();
	//udelay(2000000);

	//printf("test LEDOFF\n");
	//LEDOFF();

	//udelay(2000000);
	//printf("test LEDON\n");
	//LEDON();
	
	//tmp0 = le32_to_cpu(*(volatile u_long *)REG_PIODATA);
	//tmp1 = le32_to_cpu(*(volatile u_long *)REG_PIODIR);
	//tmp2 = le32_to_cpu(*(volatile u_long *)REG_PIOPOL);
	//printf("after LEDON:\n");	 // tmp test
	//printf("data:%x, dir:%x, pol:%x\n", tmp0, tmp1, tmp2);

	if(!(tmp0 & RESET_BTN_BIT))
	{/*reset button pressed*/
		return 1;	// disable for tmp
	}
	else 
	{
		if(!(tmp0 & EZSETUP_BTN_BIT))
		{/*EZsetup button pressed*/
			printf("\nEZSetup buootn pressed!\n");
			return 2;
		}
		else 
		{
			return 0;
		}
	}
}

void LEDON(void)
{
	led_control(0);
}

void LEDOFF(void)
{
	led_control(1);	// disable for tmp
}
